@@ -52,6 +52,7 @@ module Annex.Locations (
 	gitAnnexIndexStatus,
 	gitAnnexViewIndex,
 	gitAnnexViewLog,
+	gitAnnexMergedRefs,
 	gitAnnexIgnoredRefs,
 	gitAnnexPidFile,
 	gitAnnexPidLockFile,
@@ -357,6 +358,10 @@ gitAnnexViewIndex r = gitAnnexDir r </> "viewindex"
 gitAnnexViewLog :: Git.Repo -> FilePath
 gitAnnexViewLog r = gitAnnexDir r </> "viewlog"
 
+{- List of refs that have already been merged into the git-annex branch. -}
+gitAnnexMergedRefs :: Git.Repo -> FilePath
+gitAnnexMergedRefs r = gitAnnexDir r </> "mergedrefs"
+
 {- List of refs that should not be merged into the git-annex branch. -}
 gitAnnexIgnoredRefs :: Git.Repo -> FilePath
 gitAnnexIgnoredRefs r = gitAnnexDir r </> "ignoredrefs"
