@@ -21,6 +21,7 @@ import Utility.Parallel
 import qualified Git
 import qualified Git.Command
 import qualified Git.Ref
+import qualified Git.Merge
 import qualified Remote
 import qualified Types.Remote as Remote
 import qualified Remote.List as Remote
@@ -238,12 +239,19 @@ manualPull currentbranch remotes = do
 			)
 	haddiverged <- liftAnnex Annex.Branch.forceUpdate
 	forM_ normalremotes $ \r ->
-		liftAnnex $ Command.Sync.mergeRemote r currentbranch
+		liftAnnex $ Command.Sync.mergeRemote r currentbranch mergeConfig
 	u <- liftAnnex getUUID
 	forM_ xmppremotes $ \r ->
 		sendNetMessage $ Pushing (getXMPPClientID r) (PushRequest u)
 	return (catMaybes failed, haddiverged)
 
+mergeConfig :: [Git.Merge.MergeConfig]
+mergeConfig = 
+	[ Git.Merge.MergeNonInteractive
+	-- Pairing involves merging unrelated histories
+	, Git.Merge.MergeUnrelatedHistories
+	]
+
 {- Start syncing a remote, using a background thread. -}
 syncRemote :: Remote -> Assistant ()
 syncRemote remote = do
