@@ -9,7 +9,7 @@ module Backend.Utilities where
 
 import Data.Hash.MD5
 
-import Common.Annex
+import Annex.Common
 
 {- Generates a keyName from an input string. Takes care of sanitizing it.
  - If it's not too long, the full string is used as the keyName.
