@@ -78,9 +78,6 @@ buildFlags = filter (not . null)
 #endif
 #ifdef WITH_MAGICMIME
 	, "MagicMime"
-#endif
-#ifdef WITH_EKG
-	, "EKG"
 #endif
 	-- Always enabled now, but users may be used to seeing these flags
 	-- listed.
