@@ -5,22 +5,23 @@
  - Licensed under the GNU GPL version 3 or higher.
  -}
 
-{-# LANGUAGE CPP #-}
-
 module Command.Unannex where
 
-import Common.Annex
 import Command
 import Config
 import qualified Annex
 import Annex.Content
+import Annex.Perms
 import Annex.Content.Direct
+import Annex.Version
 import qualified Git.Command
 import qualified Git.Branch
 import qualified Git.Ref
 import qualified Git.DiffTree as DiffTree
 import Utility.CopyFile
 import Command.PreCommit (lockPreCommitHook)
+import qualified Database.Keys
+import Git.FilePath
 
 cmd :: Command
 cmd = withGlobalOptions annexedMatchingOptions $
@@ -32,7 +33,7 @@ seek :: CmdParams -> CommandSeek
 seek = wrapUnannex . (withFilesInGit $ whenAnnexed start)
 
 wrapUnannex :: Annex a -> Annex a
-wrapUnannex a = ifM isDirect
+wrapUnannex a = ifM (versionSupportsUnlockedPointers <||> isDirect)
 	( a
 	{- Run with the pre-commit hook disabled, to avoid confusing
 	 - behavior if an unannexed file is added back to git as
@@ -85,6 +86,7 @@ performIndirect file key = do
 
 cleanupIndirect :: FilePath -> Key -> CommandCleanup
 cleanupIndirect file key = do
+	Database.Keys.removeAssociatedFile key =<< inRepo (toTopFilePath file)
 	src <- calcRepo $ gitAnnexLocation key
 	ifM (Annex.getState Annex.fast)
 		( do
@@ -103,15 +105,11 @@ cleanupIndirect file key = do
 	copyfrom src = 
 		thawContent file `after` liftIO (copyFileExternal CopyAllMetaData src file)
 	hardlinkfrom src =
-#ifndef mingw32_HOST_OS
 		-- creating a hard link could fall; fall back to copying
 		ifM (liftIO $ catchBoolIO $ createLink src file >> return True)
 			( return True
 			, copyfrom src
 			)
-#else
-		copyfrom src
-#endif
 
 performDirect :: FilePath -> Key -> CommandPerform
 performDirect file key = do
