@@ -7,7 +7,6 @@
 
 module Command.Undo where
 
-import Common.Annex
 import Command
 import Config
 import Annex.Direct
@@ -72,7 +71,7 @@ perform p = do
 		f <- mkrel di
 		whenM isDirect $
 			maybe noop (`removeDirect` f)
-				=<< catKey (srcsha di) (srcmode di)
+				=<< catKey (srcsha di)
 		liftIO $ nukeFile f
 
 	forM_ adds $ \di -> do
@@ -80,6 +79,6 @@ perform p = do
 		inRepo $ Git.run [Param "checkout", Param "--", File f]
 		whenM isDirect $
 			maybe noop (`toDirect` f)
-				=<< catKey (dstsha di) (dstmode di)
+				=<< catKey (dstsha di)
 
 	next $ liftIO cleanup
