@@ -1,6 +1,6 @@
 {- git-annex command
  -
- - Copyright 2010,2015 Joey Hess <id@joeyh.name>
+ - Copyright 2010-2016 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU GPL version 3 or higher.
  -}
@@ -16,6 +16,8 @@ import Annex.Link
 import Annex.ReplaceFile
 import Utility.CopyFile
 import Utility.FileMode
+import Git.FilePath
+import qualified Database.Keys
 
 cmd :: Command
 cmd = mkcmd "unlock" "unlock files for modification"
@@ -37,14 +39,9 @@ start :: FilePath -> Key -> CommandStart
 start file key = ifM (isJust <$> isAnnexLink file)
 	( do
 		showStart "unlock" file
-		ifM (inAnnex key)
-			( ifM versionSupportsUnlockedPointers
-				( next $ performNew file key
-				, startOld file key 
-				)
-			, do
-				warning "content not present; cannot unlock"
-				next $ next $ return False
+		ifM versionSupportsUnlockedPointers
+			( next $ performNew file key
+			, startOld file key
 			)
 	, stop
 	)
@@ -52,16 +49,22 @@ start file key = ifM (isJust <$> isAnnexLink file)
 performNew :: FilePath -> Key -> CommandPerform
 performNew dest key = do
 	destmode <- liftIO $ catchMaybeIO $ fileMode <$> getFileStatus dest
-	replaceFile dest $ \tmp -> do
-		r <- linkFromAnnex key tmp destmode
-		case r of
-			LinkAnnexOk -> return ()
-			_ -> error "unlock failed"
+	replaceFile dest $ \tmp ->
+		ifM (inAnnex key)
+			( do
+				r <- linkFromAnnex key tmp destmode
+				case r of
+					LinkAnnexOk -> return ()
+					LinkAnnexNoop -> return ()
+					_ -> error "unlock failed"
+			, liftIO $ writePointerFile tmp key destmode
+			)
 	next $ cleanupNew dest key destmode
 
 cleanupNew ::  FilePath -> Key -> Maybe FileMode -> CommandCleanup
 cleanupNew dest key destmode = do
 	stagePointerFile dest destmode =<< hashPointerFile key
+	Database.Keys.addAssociatedFile key =<< inRepo (toTopFilePath dest)
 	return True
 
 startOld :: FilePath -> Key -> CommandStart
